# Crop Yield Estimation using Hierarchical Bayesian Model - partial pooling #

# Contact: Naresh Devineni, [ndevineni@ccny.cuny.edu] (Civil Engineering, The City University of New York (City College)) 
# Contact: James Rising,[jarising@gmail.com] (London School of Economics) 

# This is a demo version. 
#-------------------------------------------------------------#
# 1: set working directory 

# Your local folder should have the following files 
  # 1) "Y.csv"               | log(yield) 
  # 2) "CDI.csv"             | Cumulative Deficit Index 
  # 3) "GDD.csv"             | Growing Degree Days 
  # 4) "EDD.csv"             | Extreme Degree Days
  # 5) "bios.csv"            | bio climate 
  # 6) "script_bayes.demo.R" | the script to run the model 

# Navigate to your local folder that has the input files and "script_bayes_demo.R"
setwd(getwd()) 

#-------------------------------------------------------------#
# 2: Clear any previous R Workspace  
rm(list = ls(all = TRUE))   

#-------------------------------------------------------------#
# 3: Load Libraries and set working directory 
# If the libraries are not installed on your local machine, use "install.pacakges("name of the library")" command to  install

# Libraries #

# libraries for fitting distributions
library(locfit)
library(logspline)

# libraries for running jags in parallel
library(runjags)
library(rjags)
library(R2jags)
library(MCMCpack)
library(parallel)
library(mcmcplots)
library(superdiag)
library(coda)

#------------------------------------------------------------#
# 4: Read the input demo files
Y = read.csv("Y.csv",header=T)      # log(yield) -- units: bushels per acre
CDI = read.csv("CDI.csv",header=T)  # cumulative deficit index -- units: meters per acre
GDD = read.csv("GDD.csv",header=T)  # growing degree days -- units: degree C
EDD = read.csv("EDD.csv",header=T)  # extreme degree days -- units: degree C
bios = read.csv("bios.csv",header=T) # static bio climate variables -- units: various

#------------------------------------------------------------#
# 5: Bayesian Model
# Create the Model for JAGS #
bayes_yield_model <- function() {

  # Model Level 1 #
  for (i in 1:ncounties)
  {
    for (t in 1:nyrs)
    {
      Y[t,i] ~ dnorm(alpha[i] + beta1[i]*Technology[t,1] + beta2[i]*CDI[t,i] + beta3[i]*GDD[t,i] + beta4[i]*EDD[t,i],tau.y[i])
    }
  }
  
  # Model Level 2 #
  for (i in 1:ncounties)
  {
    alpha[i] ~ dnorm(a0 + inprod(b0,bios[i,]),tau.alpha)
    beta1[i] ~ dnorm(a1 + inprod(b1,bios[i,]),tau.beta1)
    beta2[i] ~ dnorm(a2 + inprod(b2,bios[i,]),tau.beta2)
    beta3[i] ~ dnorm(a3 + inprod(b3,bios[i,]),tau.beta3)
    beta4[i] ~ dnorm(a4 + inprod(b4,bios[i,]),tau.beta4)
  }
  
  # Priors #
  for (i in 1:ncounties)
  {
    tau.y[i] <- pow(sigma.y[i],-2)
    sigma.y[i] ~ dunif(0,100)
  }
  
  tau.alpha <- pow(sigma.alpha,-2)
  sigma.alpha ~ dunif(0,100)
  
  tau.beta1 <- pow(sigma.beta1,-2)
  sigma.beta1 ~ dunif(0,100)
  
  tau.beta2 <- pow(sigma.beta2,-2)
  sigma.beta2 ~ dunif(0,100)
  
  tau.beta3 <- pow(sigma.beta3,-2)
  sigma.beta3 ~ dunif(0,100)
  
  tau.beta4 <- pow(sigma.beta4,-2)
  sigma.beta4 ~ dunif(0,100)
  
  a0 ~ dnorm(0, 0.01)
  a1 ~ dnorm(0, 0.01)
  a2 ~ dnorm(0, 0.01)
  a3 ~ dnorm(0, 0.01)
  a4 ~ dnorm(0, 0.01)
  
  for (i in 1:nbios)
  {
    b0[i] ~ dnorm(0, 0.01)
    b1[i] ~ dnorm(0, 0.01)
    b2[i] ~ dnorm(0, 0.01)
    b3[i] ~ dnorm(0, 0.01)
    b4[i] ~ dnorm(0, 0.01)
  }

}
#------------------------------------------------------------#
# 6: Prepare data to pass it through JAGS 
ncounties = nrow(bios)
nyrs = nrow(Y)
nbios = ncol(bios)-1

# Some arrangement # 
Technology = as.matrix(Y[,1])
Y = as.matrix(Y[,2:(ncounties+1)])
CDI = as.matrix(CDI[,2:(ncounties+1)])
GDD = as.matrix(GDD[,2:(ncounties+1)])
EDD = as.matrix(EDD[,2:(ncounties+1)])
bios = as.matrix(bios[,2:(nbios+1)])

#------------------------------------------------------------#
# 7: Run the JAGS Model #
jags.data <- list('Technology' = Technology,
                  'CDI' = CDI,
                  'GDD' = GDD,
                  'EDD' = EDD,
                  'Y' = Y,
                  'ncounties' = ncounties,
                  'nyrs'=nyrs,
                  'bios'=bios,
                  'nbios' = nbios)

jags.params <- c('sigma.y',
                 'alpha','beta1','beta2','beta3','beta4',
                 'sigma.alpha','sigma.beta1','sigma.beta2','sigma.beta3','sigma.beta4',
                 'a0','a1','a2','a3','a4',
                 'b0','b1','b2','b3','b4')

jags.inits <- NULL

#=============#
# using jags  #
#=============#

# Fit the model using run.jags -- parallel #
bayes.mod.fit  <- jags.parallel(model.file = bayes_yield_model, 
                                data = jags.data, 
                                parameters.to.save = jags.params,
                                inits = jags.inits,
                                n.iter=100,
                               	jags.module = c("glm","dic"),
 				                        n.chains = 3)

attach.jags(bayes.mod.fit)

#------------------------------------------------------------#
# 8: Estimate Ymean data using the parameters  #
Ymean = matrix(NA,nrow=nyrs,ncol=ncounties)
mean.alpha = apply(alpha,2,mean)    
mean.beta1 = apply(beta1,2,mean)    
mean.beta2 = apply(beta2,2,mean)    
mean.beta3 = apply(beta3,2,mean)    
mean.beta4 = apply(beta4,2,mean)    
for (i in 1:ncounties)
  {
  for (j in 1:nyrs)
        {
          Ymean[j,i] = mean.alpha[i] + mean.beta1[i]*Technology[j,1] + mean.beta2[i]*CDI[j,i] + mean.beta3[i]*GDD[j,i] + mean.beta4[i]*EDD[j,i]
        }
  }

#------------------------------------------------------------#
# 9: Writing output files  #

# Level 1 #
write.table(sigma.y,"sigma_y.txt",row.names=F,col.names=F)
write.table(alpha,"coeff_alpha.txt",row.names=F,col.names=F)
write.table(beta1,"coeff_beta1.txt",row.names=F,col.names=F)
write.table(beta2,"coeff_beta2.txt",row.names=F,col.names=F)
write.table(beta3,"coeff_beta3.txt",row.names=F,col.names=F)
write.table(beta4,"coeff_beta4.txt",row.names=F,col.names=F)

# Level 2 #
write.table(sigma.alpha,"sigma_alpha.txt",row.names=F,col.names=F)
write.table(a0,"coeff_a0.txt",row.names=F,col.names=F)
write.table(b0,"coeff_b0.txt",row.names=F,col.names=F)

write.table(sigma.beta1,"sigma_beta1.txt",row.names=F,col.names=F)
write.table(a1,"coeff_a1.txt",row.names=F,col.names=F)
write.table(b1,"coeff_b1.txt",row.names=F,col.names=F)

write.table(sigma.beta2,"sigma_beta2.txt",row.names=F,col.names=F)
write.table(a2,"coeff_a2.txt",row.names=F,col.names=F)
write.table(b2,"coeff_b2.txt",row.names=F,col.names=F)

write.table(sigma.beta3,"sigma_beta3.txt",row.names=F,col.names=F)
write.table(a3,"coeff_a3.txt",row.names=F,col.names=F)
write.table(b3,"coeff_b3.txt",row.names=F,col.names=F)

write.table(sigma.beta4,"sigma_beta4.txt",row.names=F,col.names=F)
write.table(a4,"coeff_a4.txt",row.names=F,col.names=F)
write.table(b4,"coeff_b4.txt",row.names=F,col.names=F)

write.table(Ymean,"Ymean.txt", row.names=F,col.names=F)
#------------------------------------------------------------#

print("-------------This is the end of the Demo Code-----------")




