# Crop Yield Model using hierarchical Bayesian Estimation - partial pooling with a covariance model #

setwd(getwd())

# Libraries #
library(locfit)
library(logspline)
library(maps)
library(maptools)
library(RColorBrewer)
library(classInt)
library(rworldmap)
library(mapproj)
library(runjags)
library(rjags)
library(R2jags)
library(MCMCpack)
library(parallel)
library(mcmcplots)
library(superdiag)
library(coda)
library(geosphere)
library(corrplot)
library(rstan)
library(MASS)

#------------------------------------------------------------#
# Read data 
barley_yield = (read.csv("barley_yield_in_bu_per_acre.csv",header=T))
barley_ndi = (read.csv("barley_unit_area_deficit_in_meter.csv",header=T))
barley_gddkdd = (read.csv("barley_gddkdd.csv",header=T))
usa_fips= read.csv("fips_usa.csv")
usa_bioclims= read.csv("bioclims.csv")
usa_irrfrac= read.csv("irrigation_fraction.csv")
county_centroids = read.csv("centroids.csv",header=T)
barley_yield_for_impute = read.table("Ypredict.txt",header=F)

#------------------------------------------------------------#
# Arranging the data for consistency 

# Arranging GDD and KDD data #
ndistricts = nrow(barley_yield) # 3111 districts
nyrs = ncol(barley_yield)-4     # 61 years from 1949 - 2009

barley_gdd = matrix(NA,nrow=nyrs,ncol=ndistricts)
barley_kdd = matrix(NA,nrow=nyrs,ncol=ndistricts)
for (i in 1:ndistricts)
  {
    fip = usa_fips[i,1]
    dummy = which(fip==barley_gddkdd[,1])
    if((length(dummy)>0)&(nrow(barley_gddkdd[dummy,])==61)){
    barley_gdd[,i] = barley_gddkdd[dummy,3]
    barley_kdd[,i] = barley_gddkdd[dummy,4]}
  }

# Arranging the bioclims data #
nbios = 19 # 19 bioclimate attributes can be found here: http://www.worldclim.org/bioclim
bios = matrix(NA,nrow=ndistricts,ncol=nbios)
for (i in 1:nrow(usa_bioclims))
{
  fip_bio = usa_bioclims[i,]$fips
  index = which(fip_bio==usa_fips[,1])
  bios[index,1] = usa_bioclims[i,]$bio1mean
  bios[index,2] = usa_bioclims[i,]$bio2mean
  bios[index,3] = usa_bioclims[i,]$bio3mean
  bios[index,4] = usa_bioclims[i,]$bio4mean
  bios[index,5] = usa_bioclims[i,]$bio5mean
  bios[index,6] = usa_bioclims[i,]$bio6mean
  bios[index,7] = usa_bioclims[i,]$bio7mean
  bios[index,8] = usa_bioclims[i,]$bio8mean
  bios[index,9] = usa_bioclims[i,]$bio9mean
  bios[index,10] = usa_bioclims[i,]$bio10mean
  bios[index,11] = usa_bioclims[i,]$bio11mean
  bios[index,12] = usa_bioclims[i,]$bio12mean
  bios[index,13] = usa_bioclims[i,]$bio13mean
  bios[index,14] = usa_bioclims[i,]$bio14mean
  bios[index,15] = usa_bioclims[i,]$bio15mean
  bios[index,16] = usa_bioclims[i,]$bio16mean
  bios[index,17] = usa_bioclims[i,]$bio17mean
  bios[index,18] = usa_bioclims[i,]$bio18mean
  bios[index,19] = usa_bioclims[i,]$bio19mean  
}

# There are some counties with missing bioclims data due to country boundary issues. We identify them and replace  with it with a random county in the state + some noise
dum = which(is.na(bios[,1])==TRUE)
for (i in 1:length(dum))
{
  dummy = which(usa_fips[,4]==usa_fips[dum[i],4])
  dummy2 = intersect(dummy,dum)
  dummy3 = NA
  for (j in 1:length(dummy2))
    {
      dummy3 = c(dummy3,which(dummy==dummy2[j]))
    }
  dummy3 = na.omit(dummy3) 
  random_county_from_state = sample(dummy[-dummy3],1)
  bios[dum[i],]=bios[random_county_from_state,] + rnorm(nbios,0,0.5)
}

### Irrigation Fraction ###
irrigation_fraction = matrix(NA,nrow=ndistricts,ncol=1)
for (i in 1:ndistricts)
{
  fip = usa_fips[i,1]
  dummy4 = which(fip==usa_irrfrac[,1])
  irrigation_fraction[i,1] = usa_irrfrac[dummy4,2]
}

bios = cbind(bios[,1],bios[,3],bios[,4],bios[,12],bios[,15],irrigation_fraction)
nbios = ncol(bios)

### Distance Matrix Creation ###
lat = county_centroids$Y
lon = county_centroids$X
xy = cbind(lon,lat)
dist_counties = round(0.001*distm(xy,fun=distHaversine),2) # distance matrix in KM.

#------------------------------------------------------------#
# 7: Prepare the data for JAGS/STAN #

nsites = ndistricts
D = dist_counties

Y = matrix(NA,nrow=nyrs,ncol=nsites)
#Technology = as.matrix(1949:2009)               # Time trend
Technology = as.matrix(1:61)               # Time trend
NDI = matrix(NA,nrow=nyrs,ncol=nsites) 
GDD = matrix(NA,nrow=nyrs,ncol=nsites) 
KDD = matrix(NA,nrow=nyrs,ncol=nsites) 

for (i in 1:ndistricts)
{
  Y[,i] = log(as.numeric(barley_yield[i,4:64])) ## yield data from 1949:2009
  NDI[,i] = (as.numeric(barley_ndi[i,5:65])) ## NDI data from 1949:2009
  GDD[,i] = (as.numeric(barley_gdd[,i])) ## GDD data from 1949:2009
  KDD[,i] = (as.numeric(barley_kdd[,i])) ## KDD data from 1949:2009
}

# Replace 0 yield with NA
Y[which(Y==-Inf)] = NA

# Fill NAs in missing GDD value with random county from the State #
for (i in 1:nyrs)
{
  dummy = which(is.na(GDD[i,])=="TRUE")
  
  for (j in 1:length(dummy))
  {
    dummy2 = which(usa_fips[,4]==usa_fips[dummy[j],4])
    dummy3 = intersect(dummy,dummy2)
    
    dummy4 = NA
    for (k in 1:length(dummy3))
    {
      dummy4 = c(dummy4,which(dummy2==dummy3[k]))
    }
    dummy4 = na.omit(dummy4) 
    
    random_county_from_state = sample(dummy2[-dummy4],1)
    GDD[i,dummy[j]] = jitter(GDD[i,random_county_from_state]) 
    KDD[i,dummy[j]] = jitter(KDD[i,random_county_from_state])
  }
}

# Impute NAs in Y to create a complete data matrix (required for MVNORM in JAGS)
# Imputation is done only for counties that have >= 10 years. Counties < 10years are dropped from the model 

nmiss_counties = apply(Y,2,function(x){length(which(is.na(x)==TRUE))})

county_index_for_dropping = which(nmiss_counties > 50)

# full imputation before dropping # 
Y_imputed = Y

for(i in 1:nsites)
  {
    if(nmiss_counties[i]>0){
      ind = which(is.na(Y[,i])==TRUE)
      Y_imputed[ind,i] = barley_yield_for_impute[ind,i]
      print(i)
    }
}

#------------------------------------------------------------#
# 8: Bayesian Model

# STAN model
stan_yield_model <- '
data {
  int<lower=1> Nc; // number of counties 
  int<lower=1> Ny; // number of years of data 
  int<lower=1> Nb; // number of bios predictors

  real Technology[Ny, 1]; // matrix of Technology predictor
  real NDI[Ny, Nc];        // matrix of NDI predictor
  real GDD[Ny, Nc];        // matrix of GDD predictor
  real KDD[Ny, Nc];        // matrix of KDD predictor
  
  vector[Nc] y[Ny]; // matrix of y
  
  matrix [Nc, Nb] bios;  // Matrix (row, col) of bios predictor
  
  matrix [Nc, Nc] D; // Matrix of the distance
}

parameters {
  real a0; // intercept for the model of alpha 
  real a1; // intercept for the model of beta_tech
  real a2; // intercept for the model of beta_ndi
  real a3; // intercept for the model of beta_gdd
  real a4; // intercept for the model of beta_kdd 
  
  vector[Nb] b0; // Vector of the regression coefficients for the model of alpha
  vector[Nb] b1; // Vector of the regression coefficients for the model of beta_tech
  vector[Nb] b2; // Vector of the regression coefficients for the model of beta_ndi
  vector[Nb] b3; // Vector of the regression coefficients for the model of beta_gdd
  vector[Nb] b4; // Vector of the regression coefficients for the model of beta_kdd
  
  real<lower=0> sigma_alpha; // standard deviation for the model of alpha 
  real<lower=0> sigma_beta_tech; // standard deviation for the model of beta_tech
  real<lower=0> sigma_beta_ndi; // standard deviation for the model of beta_ndi
  real<lower=0> sigma_beta_gdd; // standard deviation for the model of beta_gdd
  real<lower=0> sigma_beta_kdd; // standard deviation for the model of beta_kdd 
  
  real<lower=0> delta; // scale parameter for covariance model 
  real<lower=0> lambda; // exponent parameter for covariance model

  vector[Nc] alpha;
  vector[Nc] beta_tech;
  vector[Nc] beta_ndi;
  vector[Nc] beta_gdd;
  vector[Nc] beta_kdd;

  vector<lower=0>[Nc] sigma_y;
}

transformed parameters {
  cov_matrix[Nc] Sigma;

  // model for the Sigma
  
  for (m in 1:Nc){
    Sigma[m, m] <- sigma_y[m]*sigma_y[m];
  }

    for (m in 1:(Nc-1)) {
      for (n in (m+1):Nc) {
        Sigma[m, n] <- delta*exp(-lambda*D[m, n]);
        Sigma[n, m] <- Sigma[m, n];
      }
    }
}

model {
  vector[Nc] mu[Ny]; // yhat matrix 

  vector[Nc] mu_alpha;      // mean of alpha
  vector[Nc] mu_beta_tech;  // mean of beta_tech
  vector[Nc] mu_beta_ndi;   // mean of beta_ndi
  vector[Nc] mu_beta_gdd;   // mean of beta_gdd
  vector[Nc] mu_beta_kdd;   // mean of beta_kdd
  
  // model for the mean
    for (n in 1:Ny){
      for (k in 1:Nc){
        mu[n,k] <- alpha[k] + beta_tech[k]*Technology[n,1] + beta_ndi[k]*NDI[n,k] + beta_gdd[k]*GDD[n,k] + beta_kdd[k]*KDD[n,k];
          }
      }

  // Model for the mean of the regression coefficients 
    mu_alpha <- a0 + bios * b0;
    mu_beta_tech <- a1 + bios * b1;
    mu_beta_ndi <- a2 + bios * b2;
    mu_beta_gdd <- a3 + bios * b3;
    mu_beta_kdd <- a4 + bios * b4;

// priors 
  delta ~ cauchy(0, 100);
  lambda ~ cauchy(0, 1);
  sigma_y ~ cauchy(0, 5);

  a0 ~ normal(0, 5); 
  a1 ~ normal(0, 5); 
  a2 ~ normal(0, 5); 
  a3 ~ normal(0, 5); 
  a4 ~ normal(0, 5); 
 
  b0 ~ normal(0, 5); 
  b1 ~ normal(0, 5); 
  b2 ~ normal(0, 5); 
  b3 ~ normal(0, 5); 
  b4 ~ normal(0, 5); 

  sigma_alpha ~ cauchy(0, 5);
  sigma_beta_tech ~ cauchy(0, 5);
  sigma_beta_ndi ~ cauchy(0, 5);
  sigma_beta_gdd ~ cauchy(0, 5);
  sigma_beta_kdd ~ cauchy(0, 5);

// models 
  alpha ~ normal(mu_alpha, sigma_alpha);
  beta_tech ~ normal(mu_beta_tech, sigma_beta_tech);
  beta_ndi ~ normal(mu_beta_ndi, sigma_beta_ndi);
  beta_gdd ~ normal(mu_beta_gdd, sigma_beta_gdd);
  beta_kdd ~ normal(mu_beta_kdd, sigma_beta_kdd);

  y ~ multi_normal(mu, Sigma);
}
'

print ("Stage 7 - model loaded")

#------------------------------------------------------------#
# 9: Run the JAGS/STAN Model # 

NDI_imputed_for_model = NDI[,-county_index_for_dropping]
GDD_imputed_for_model = GDD[,-county_index_for_dropping]
KDD_imputed_for_model = KDD[,-county_index_for_dropping]
Y_imputed_for_model = Y_imputed[,-county_index_for_dropping]
bios_imputed_for_model = bios[-county_index_for_dropping,]
D_imputed_for_model = D[-county_index_for_dropping,-county_index_for_dropping]

sample_counties = 1:10

stan.data <- list(Nc = ncol(Y_imputed_for_model[,sample_counties]),
                  Ny = nrow(Y_imputed_for_model[,sample_counties]),
                  Nb = ncol(bios),
                  Technology = Technology,
                  NDI = NDI_imputed_for_model[,sample_counties],
                  GDD = GDD_imputed_for_model[,sample_counties],
                  KDD = KDD_imputed_for_model[,sample_counties],
                  bios = bios_imputed_for_model[sample_counties,],
                  D = D[sample_counties,sample_counties],
                  y = Y_imputed_for_model[,sample_counties])

stan.params <- c("Sigma",
                  "alpha", "beta_tech", "beta_ndi", "beta_gdd", "beta_kdd",
                  "a0", "a1", "a2", "a3", "a4",
                  "b0", "b1", "b2", "b3", "b4",
                  "sigma_alpha", "sigma_beta_tech", "sigma_beta_ndi", "sigma_beta_gdd", "sigma_beta_kdd",
                  "delta", "lambda")

# function form  with an argument named `chain_id`
initf <- function(chain_id = 1) {
  # cat("chain_id =", chain_id, "\n")
  list(a0 = rnorm(1), a1 = rnorm(1), a2 = rnorm(1), a3 = rnorm(1), a4 = rnorm(1),
       b0 = rnorm(nbios), b1 = rnorm(nbios), b2 = rnorm(nbios), b3 = rnorm(nbios), b4 = rnorm(nbios),
       delta = runif(1,0,10), lambda = runif(1,0,1),
       sigma_alpha = runif(1,0,10),
       sigma_beta_tech = runif(1,0,10),
       sigma_beta_ndi = runif(1,0,10),
       sigma_beta_gdd = runif(1,0,10),
       sigma_beta_kdd = runif(1,0,10),
       sigma_y = runif(length(sample_counties),0,10),
       chain = chain_id)
}

# generate a list of lists to specify initial values
n_chains <- 3
init_ll <- lapply(1:n_chains, function(id) initf(chain_id = id))

#options(mc.cores = parallel::detectCores())
fit <- stan(model_code = stan_yield_model, 
            data = stan.data, pars = stan.params,
            algorithm = "NUTS", init = init_ll, iter = 100, chains = n_chains, cores = n_chains, verbose=TRUE)

posterior_data = extract(fit)


pdf("traceplots.pdf",width=24,height=24)
traceplot(fit,pars="alpha")
traceplot(fit,pars="beta_tech")
traceplot(fit,pars="beta_ndi")
traceplot(fit,pars="beta_gdd")
traceplot(fit,pars="beta_kdd")
#traceplot(fit,pars="Sigma")
traceplot(fit,pars=c("a0","a1", "a2", "a3", "a4"))
traceplot(fit,pars=c("b0","b1", "b2", "b3", "b4"))
traceplot(fit,pars=c("sigma_alpha","sigma_beta_tech", "sigma_beta_ndi", "sigma_beta_gdd", "sigma_beta_kdd"))
traceplot(fit,pars=c("delta","lambda"))
dev.off()

#save.image("output_Nc100It10000.RData")




